@@ -4,6 +4,7 @@ from click_default_group import DefaultGroup  # type: ignore
 from datetime import datetime
 import hashlib
 import pathlib
+from runpy import run_module
 import sqlite_utils
 from sqlite_utils.db import AlterError, BadMultiValues, DescIndex, NoTable
 from sqlite_utils.utils import maximize_csv_field_size_limit
@@ -23,6 +24,7 @@ from .utils import (
     chunks,
     file_progress,
     find_spatialite,
+    flatten as _flatten,
     sqlite3,
     decode_base64_values,
     progressbar,
@@ -953,14 +955,16 @@ def insert_upsert_implementation(
         decoded = io.TextIOWrapper(file, encoding=encoding)
 
     tracker = None
-    if csv or tsv:
-        if sniff:
-            # Read first 2048 bytes and use that to detect
-            first_bytes = sniff_buffer.peek(2048)
-            dialect = csv_std.Sniffer().sniff(first_bytes.decode(encoding, "ignore"))
-        else:
-            dialect = "excel-tab" if tsv else "excel"
-        with file_progress(decoded, silent=silent) as decoded:
+    with file_progress(decoded, silent=silent) as decoded:
+        if csv or tsv:
+            if sniff:
+                # Read first 2048 bytes and use that to detect
+                first_bytes = sniff_buffer.peek(2048)
+                dialect = csv_std.Sniffer().sniff(
+                    first_bytes.decode(encoding, "ignore")
+                )
+            else:
+                dialect = "excel-tab" if tsv else "excel"
             csv_reader_args = {"dialect": dialect}
             if delimiter:
                 csv_reader_args["delimiter"] = delimiter
@@ -977,24 +981,24 @@ def insert_upsert_implementation(
             if detect_types:
                 tracker = TypeTracker()
                 docs = tracker.wrap(docs)
-    elif lines:
-        docs = ({"line": line.strip()} for line in decoded)
-    elif text:
-        docs = ({"text": decoded.read()},)
-    else:
-        try:
-            if nl:
-                docs = (json.loads(line) for line in decoded if line.strip())
-            else:
-                docs = json.load(decoded)
-                if isinstance(docs, dict):
-                    docs = [docs]
-        except json.decoder.JSONDecodeError:
-            raise click.ClickException(
-                "Invalid JSON - use --csv for CSV or --tsv for TSV files"
-            )
-        if flatten:
-            docs = (dict(_flatten(doc)) for doc in docs)
+        elif lines:
+            docs = ({"line": line.strip()} for line in decoded)
+        elif text:
+            docs = ({"text": decoded.read()},)
+        else:
+            try:
+                if nl:
+                    docs = (json.loads(line) for line in decoded if line.strip())
+                else:
+                    docs = json.load(decoded)
+                    if isinstance(docs, dict):
+                        docs = [docs]
+            except json.decoder.JSONDecodeError:
+                raise click.ClickException(
+                    "Invalid JSON - use --csv for CSV or --tsv for TSV files"
+                )
+            if flatten:
+                docs = (_flatten(doc) for doc in docs)
 
     if convert:
         variable = "row"
@@ -1076,15 +1080,6 @@ def insert_upsert_implementation(
         db[table].transform(types=tracker.types)
 
 
-def _flatten(d):
-    for key, value in d.items():
-        if isinstance(value, dict):
-            for key2, value2 in _flatten(value):
-                yield key + "_" + key2, value2
-        else:
-            yield key, value
-
-
 def _find_variables(tb, vars):
     to_find = list(vars)
     found = {}
@@ -1842,7 +1837,7 @@ def memory(
             tracker = TypeTracker()
             rows = tracker.wrap(rows)
         if flatten:
-            rows = (dict(_flatten(row)) for row in rows)
+            rows = (_flatten(row) for row in rows)
         db[csv_table].insert_all(rows, alter=True)
         if tracker is not None:
             db[csv_table].transform(types=tracker.types)
@@ -2655,6 +2650,30 @@ def _analyze(db, tables, columns, save):
         click.echo(details)
 
 
+@cli.command()
+@click.argument("packages", nargs=-1, required=True)
+@click.option(
+    "-U", "--upgrade", is_flag=True, help="Upgrade packages to latest version"
+)
+def install(packages, upgrade):
+    """Install packages from PyPI into the same environment as sqlite-utils"""
+    args = ["pip", "install"]
+    if upgrade:
+        args += ["--upgrade"]
+    args += list(packages)
+    sys.argv = args
+    run_module("pip", run_name="__main__")
+
+
+@cli.command()
+@click.argument("packages", nargs=-1, required=True)
+@click.option("-y", "--yes", is_flag=True, help="Don't ask for confirmation")
+def uninstall(packages, yes):
+    """Uninstall Python packages from the sqlite-utils environment"""
+    sys.argv = ["pip", "uninstall"] + list(packages) + (["-y"] if yes else [])
+    run_module("pip", run_name="__main__")
+
+
 def _generate_convert_help():
     help = textwrap.dedent(
         """
