@@ -7,6 +7,7 @@ import io
 import itertools
 import json
 import os
+import sys
 from . import recipes
 from typing import Dict, cast, BinaryIO, Iterable, Optional, Tuple, Type
 
@@ -29,6 +30,24 @@ SPATIALITE_PATHS = (
     "/usr/local/lib/mod_spatialite.dylib",
 )
 
+# Mainly so we can restore it if needed in the tests:
+ORIGINAL_CSV_FIELD_SIZE_LIMIT = csv.field_size_limit()
+
+
+def maximize_csv_field_size_limit():
+    """
+    Increase the CSV field size limit to the maximum possible.
+    """
+    # https://stackoverflow.com/a/15063941
+    field_size_limit = sys.maxsize
+
+    while True:
+        try:
+            csv.field_size_limit(field_size_limit)
+            break
+        except OverflowError:
+            field_size_limit = int(field_size_limit / 10)
+
 
 def find_spatialite() -> Optional[str]:
     """
@@ -136,6 +155,11 @@ class UpdateWrapper:
             self._update(len(line))
             yield line
 
+    def read(self, size=-1):
+        data = self._wrapped.read(size)
+        self._update(len(data))
+        return data
+
 
 @contextlib.contextmanager
 def file_progress(file, silent=False, **kwargs):
@@ -171,12 +195,85 @@ class RowsFromFileBadJSON(RowsFromFileError):
     pass
 
 
+class RowError(Exception):
+    pass
+
+
+def _extra_key_strategy(
+    reader: Iterable[dict],
+    ignore_extras: Optional[bool] = False,
+    extras_key: Optional[str] = None,
+) -> Iterable[dict]:
+    # Logic for handling CSV rows with more values than there are headings
+    for row in reader:
+        # DictReader adds a 'None' key with extra row values
+        if None not in row:
+            yield row
+        elif ignore_extras:
+            # ignoring row.pop(none) because of this issue:
+            # https://github.com/simonw/sqlite-utils/issues/440#issuecomment-1155358637
+            row.pop(None)  # type: ignore
+            yield row
+        elif not extras_key:
+            extras = row.pop(None)  # type: ignore
+            raise RowError(
+                "Row {} contained these extra values: {}".format(row, extras)
+            )
+        else:
+            row[extras_key] = row.pop(None)  # type: ignore
+            yield row
+
+
 def rows_from_file(
     fp: BinaryIO,
     format: Optional[Format] = None,
     dialect: Optional[Type[csv.Dialect]] = None,
     encoding: Optional[str] = None,
+    ignore_extras: Optional[bool] = False,
+    extras_key: Optional[str] = None,
 ) -> Tuple[Iterable[dict], Format]:
+    """
+    Load a sequence of dictionaries from a file-like object containing one of four different formats.
+
+    .. code-block:: python
+
+        from sqlite_utils.utils import rows_from_file
+        import io
+
+        rows, format = rows_from_file(io.StringIO("id,name\\n1,Cleo")))
+        print(list(rows), format)
+        # Outputs [{'id': '1', 'name': 'Cleo'}] Format.CSV
+
+    This defaults to attempting to automatically detect the format of the data, or you can pass in an
+    explicit format using the format= option.
+
+    Returns a tuple of ``(rows_generator, format_used)`` where ``rows_generator`` can be iterated over
+    to return dictionaries, while ``format_used`` is a value from the ``sqlite_utils.utils.Format`` enum:
+
+    .. code-block:: python
+
+        class Format(enum.Enum):
+            CSV = 1
+            TSV = 2
+            JSON = 3
+            NL = 4
+
+    If a CSV or TSV file includes rows with more fields than are declared in the header a
+    ``sqlite_utils.utils.RowError`` exception will be raised when you loop over the generator.
+
+    You can instead ignore the extra data by passing ``ignore_extras=True``.
+
+    Or pass ``extras_key="rest"`` to put those additional values in a list in a key called ``rest``.
+
+    :param fp: a file-like object containing binary data
+    :param format: the format to use - omit this to detect the format
+    :param dialect: the CSV dialect to use - omit this to detect the dialect
+    :param encoding: the character encoding to use when reading CSV/TSV data
+    :param ignore_extras: ignore any extra fields on rows
+    :param extras_key: put any extra fields in a list with this key
+    """
+    if ignore_extras and extras_key:
+        raise ValueError("Cannot use ignore_extras= and extras_key= together")
     if format == Format.JSON:
         decoded = json.load(fp)
         if isinstance(decoded, dict):
@@ -193,12 +290,13 @@ def rows_from_file(
             reader = csv.DictReader(decoded_fp, dialect=dialect)
         else:
             reader = csv.DictReader(decoded_fp)
-        return reader, Format.CSV
+        return _extra_key_strategy(reader, ignore_extras, extras_key), Format.CSV
     elif format == Format.TSV:
+        rows = rows_from_file(
+            fp, format=Format.CSV, dialect=csv.excel_tab, encoding=encoding
+        )[0]
         return (
-            rows_from_file(
-                fp, format=Format.CSV, dialect=csv.excel_tab, encoding=encoding
-            )[0],
+            _extra_key_strategy(rows, ignore_extras, extras_key),
             Format.TSV,
         )
     elif format is None:
@@ -212,18 +310,47 @@ def rows_from_file(
             dialect = csv.Sniffer().sniff(
                 first_bytes.decode(encoding or "utf-8-sig", "ignore")
             )
-            return rows_from_file(
+            rows, _ = rows_from_file(
                 buffered, format=Format.CSV, dialect=dialect, encoding=encoding
             )
+            # Make sure we return the format we detected
+            format = Format.TSV if dialect.delimiter == "\t" else Format.CSV
+            return _extra_key_strategy(rows, ignore_extras, extras_key), format
     else:
         raise RowsFromFileError("Bad format")
 
 
 class TypeTracker:
+    """
+    Wrap an iterator of dictionaries and keep track of which SQLite column
+    types are the most likely fit for each of their keys.
+
+    Example usage:
+
+    .. code-block:: python
+
+        from sqlite_utils.utils import TypeTracker
+        import sqlite_utils
+
+        db = sqlite_utils.Database(memory=True)
+        tracker = TypeTracker()
+        rows = [{"id": "1", "name": "Cleo", "id": "2", "name": "Cardi"}]
+        db["creatures"].insert_all(tracker.wrap(rows))
+        print(tracker.types)
+        # Outputs {'id': 'integer', 'name': 'text'}
+        db["creatures"].transform(types=tracker.types)
+    """
+
     def __init__(self):
         self.trackers = {}
 
-    def wrap(self, iterator):
+    def wrap(self, iterator: Iterable[dict]) -> Iterable[dict]:
+        """
+        Use this to loop through an existing iterator, tracking the column types
+        as part of the iteration.
+
+        :param iterator: The iterator to wrap
+        """
         for row in iterator:
             for key, value in row.items():
                 tracker = self.trackers.setdefault(key, ValueTracker())
@@ -231,7 +358,11 @@ class TypeTracker:
             yield row
 
     @property
-    def types(self):
+    def types(self) -> Dict[str, str]:
+        """
+        A dictionary mapping column names to their detected types. This can be passed
+        to the ``db[table_name].transform(types=tracker.types)`` method.
+        """
         return {key: tracker.guessed_type for key, tracker in self.trackers.items()}
 
 
@@ -261,7 +392,7 @@ class ValueTracker:
         except (ValueError, TypeError):
             return False
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.guessed_type + ": possibilities = " + repr(self.couldbe)
 
     @property
@@ -306,12 +437,11 @@ def progressbar(*args, **kwargs):
 
 
 def _compile_code(code, imports, variable="value"):
-    locals = {}
     globals = {"r": recipes, "recipes": recipes}
     # If user defined a convert() function, return that
     try:
-        exec(code, globals, locals)
-        return locals["convert"]
+        exec(code, globals)
+        return globals["convert"]
     except (AttributeError, SyntaxError, NameError, KeyError, TypeError):
         pass
 
@@ -338,11 +468,17 @@ def _compile_code(code, imports, variable="value"):
 
     for import_ in imports:
         globals[import_.split(".")[0]] = __import__(import_)
-    exec(code_o, globals, locals)
-    return locals["fn"]
+    exec(code_o, globals)
+    return globals["fn"]
 
 
-def chunks(sequence, size):
+def chunks(sequence: Iterable, size: int) -> Iterable[Iterable]:
+    """
+    Iterate over chunks of the sequence of the given size.
+
+    :param sequence: Any Python iterator
+    :param size: The size of each chunk
+    """
     iterator = iter(sequence)
     for item in iterator:
         yield itertools.chain([item], itertools.islice(iterator, size - 1))
@@ -377,3 +513,21 @@ def hash_record(record: Dict, keys: Optional[Iterable[str]] = None):
             "utf8"
         )
     ).hexdigest()
+
+
+def _flatten(d):
+    for key, value in d.items():
+        if isinstance(value, dict):
+            for key2, value2 in _flatten(value):
+                yield key + "_" + key2, value2
+        else:
+            yield key, value
+
+
+def flatten(row: dict) -> dict:
+    """
+    Turn a nested dict e.g. ``{"a": {"b": 1}}`` into a flat dict: ``{"a_b": 1}``
+
+    :param row: A Python dictionary, optionally with nested dictionaries
+    """
+    return dict(_flatten(row))
