@@ -14,6 +14,24 @@ def test_insert_m2m_single(fresh_db):
     assert [{"humans_id": 1, "dogs_id": 1}] == list(dogs_humans.rows)
 
 
+def test_insert_m2m_alter(fresh_db):
+    dogs = fresh_db["dogs"]
+    dogs.insert({"id": 1, "name": "Cleo"}, pk="id").m2m(
+        "humans", {"id": 1, "name": "Natalie D"}, pk="id"
+    )
+    dogs.update(1).m2m(
+        "humans", {"id": 2, "name": "Simon W", "nerd": True}, pk="id", alter=True
+    )
+    assert list(fresh_db["humans"].rows) == [
+        {"id": 1, "name": "Natalie D", "nerd": None},
+        {"id": 2, "name": "Simon W", "nerd": 1},
+    ]
+    assert list(fresh_db["dogs_humans"].rows) == [
+        {"humans_id": 1, "dogs_id": 1},
+        {"humans_id": 2, "dogs_id": 1},
+    ]
+
+
 def test_insert_m2m_list(fresh_db):
     dogs = fresh_db["dogs"]
     dogs.insert({"id": 1, "name": "Cleo"}, pk="id").m2m(
@@ -43,6 +61,46 @@ def test_insert_m2m_list(fresh_db):
     ] == dogs_humans.foreign_keys
 
 
+def test_insert_m2m_iterable(fresh_db):
+    iterable_records = ({"id": 1, "name": "Phineas"}, {"id": 2, "name": "Ferb"})
+
+    def iterable():
+        for record in iterable_records:
+            yield record
+
+    platypuses = fresh_db["platypuses"]
+    platypuses.insert({"id": 1, "name": "Perry"}, pk="id").m2m(
+        "humans",
+        iterable(),
+        pk="id",
+    )
+
+    assert {"platypuses", "humans", "humans_platypuses"} == set(fresh_db.table_names())
+    humans = fresh_db["humans"]
+    humans_platypuses = fresh_db["humans_platypuses"]
+    assert [
+        {"humans_id": 1, "platypuses_id": 1},
+        {"humans_id": 2, "platypuses_id": 1},
+    ] == list(humans_platypuses.rows)
+    assert [{"id": 1, "name": "Phineas"}, {"id": 2, "name": "Ferb"}] == list(
+        humans.rows
+    )
+    assert [
+        ForeignKey(
+            table="humans_platypuses",
+            column="platypuses_id",
+            other_table="platypuses",
+            other_column="id",
+        ),
+        ForeignKey(
+            table="humans_platypuses",
+            column="humans_id",
+            other_table="humans",
+            other_column="id",
+        ),
+    ] == humans_platypuses.foreign_keys
+
+
 def test_m2m_with_table_objects(fresh_db):
     dogs = fresh_db.table("dogs", pk="id")
     humans = fresh_db.table("humans", pk="id")
